/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             I/O multiplexing
*  SCOPE :              Implements multiplexing for sockrpc
*  FILE :               pollset.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      February 1, 1994
*  COPYRIGHTS :        (C) Apic Systemes, 1994
*  HISTORY :
*
******************************************************************************/

/* IMPORTS *******************************************************************/

#ifdef __GNUG__
#pragma interface
#endif

#include <sys/types.h>
#include <poll.h>

#include <stdlib.h>

#ifndef _pollset_HXX
#define _pollset_HXX

#ifndef sockrpc_HXX
#include "sockrpc.hxx"
#endif

#include "list.hxx"

#define MAX_SOCK_FD 64
			       
class SvcServer ;

typedef enum { SVC_CONNECT, SVC_DATA, SVC_DISCONNECT } SvcEvent ;

class PollSet {
  private :

  int                  max ;
  int                  size ;
  struct pollfd        set[MAX_SOCK_FD] ;
  ListPtr<SvcServer>   svc_l ;

  void                 setfd(int) ;
  void                 unsetfd(int) ;
  
  public:
  
  PollSet();
  ~PollSet() ;
  int                  testfd(int fd) { return (set[fd].fd != -1) ; }
  void                 disconnect(SvcServer* s) { unsetfd(s->fd()) ; }
  PollSet&             operator += (SvcServer* s) ;
  PollSet&             operator -= (SvcServer* s) ;
  SvcServer*           operator [] (int) ;
  SvcEvent             findEvent(SvcServer* svc) ;
  int                  replyEvent(int fd) ;
  int                  exec(int t_out = -1) ;
  void                 execLoop(int t_out = -1) ;
};

#endif

